import Flutter
import UIKit
import Gist

enum ChannelName {
    static let methods = "gist_flutter"
    static let events = "gist_flutter_events"
}

public class SwiftGistFlutterPlugin: NSObject, FlutterPlugin {
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: ChannelName.methods, binaryMessenger: registrar.messenger())
        let eventsChannel = FlutterEventChannel(name: ChannelName.events, binaryMessenger: registrar.messenger())
        registrar.addMethodCallDelegate(SwiftGistFlutterPlugin(), channel: channel)
        eventsChannel.setStreamHandler(GistEventHandler())
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "setup":
            if let param = call.arguments as? String {
                Gist.shared.setup(organizationId: param)
            }
        case "setUserToken":
            if let param = call.arguments as? String {
                Gist.shared.setUserToken(param)
            }
        case "clearUserToken":
            Gist.shared.clearUserToken()
        case "setCurrentRoute":
            if let param = call.arguments as? String {
                Gist.shared.setCurrentRoute(param)
            }
        case "subscribeToTopic":
            if let param = call.arguments as? String {
                Gist.shared.subscribeToTopic(param)
            }
        case "unsubscribeFromTopic":
            if let param = call.arguments as? String {
                Gist.shared.unsubscribeFromTopic(param)
            }
        case "clearTopics":
            Gist.shared.clearTopics()
        default:
            result(FlutterMethodNotImplemented)
        }
    }
}

class GistEventHandler: NSObject, FlutterStreamHandler, GistDelegate {
    private var eventSink: FlutterEventSink?
    
    public func onListen(withArguments arguments: Any?,
                         eventSink: @escaping FlutterEventSink) -> FlutterError? {
        self.eventSink = eventSink
        Gist.shared.delegate = self
        return nil
    }
    
    public func onCancel(withArguments arguments: Any?) -> FlutterError? {
        Gist.shared.delegate = nil
        eventSink = nil
        return nil
    }
    
    public func messageShown(message: Message) {
        guard let eventSink = eventSink else {
            return
        }
        let eventDict = prepareMessage(eventName: "onMessageShown", message: message)
        eventSink(eventDict)
    }
    
    public func messageDismissed(message: Message) {
        guard let eventSink = eventSink else {
            return
        }
        let eventDict = prepareMessage(eventName: "onMessageDismissed", message: message)
        eventSink(eventDict)
    }
    
    public func messageError(message: Message) {
        guard let eventSink = eventSink else {
            return
        }
        let eventDict = prepareMessage(eventName: "onError", message: message)
        eventSink(eventDict)
    }
    
    public func action(message: Message, currentRoute: String, action: String) {
        guard let eventSink = eventSink else {
            return
        }
        var eventDict = prepareMessage(eventName: "onAction", message: message)
        eventDict["currentRoute"] = currentRoute
        eventDict["action"] = action
        eventSink(eventDict)
    }
    
    public func embedMessage(message: Message, elementId: String) {
        // Not implemented in Flutter
    }
    
    private func prepareMessage(eventName: String, message: Message) -> [String: String?] {
        let messageDict: [String: String?] =
                        ["eventName": eventName,
                         "messageId": message.messageId,
                         "instanceId": message.instanceId,
                         "queueId": message.queueId]
        return messageDict;
    }
}
