import 'dart:async';
import 'dart:developer';
import 'package:flutter/services.dart';
import 'package:gist_flutter/gist_action.dart';
import 'package:gist_flutter/gist_message.dart';

class Gist {
  static const MethodChannel _channel = const MethodChannel('gist_flutter');
  static const EventChannel _eventChannel = const EventChannel('gist_flutter_events');
  static var eventListeners = <GistDelegate?>[];

  static Future<String?> get platformVersion async {
    final String? version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  static setup(organizationId) {
    _channel.invokeListMethod("setup", organizationId);
    _eventChannel.receiveBroadcastStream().listen(_onEvent, onError: _onError);
  }

  // User

  static setUserToken(userToken) {
    _channel.invokeListMethod("setUserToken", userToken);
  }

  static clearUserToken() {
    _channel.invokeListMethod("clearUserToken");
  }

  // Routing

  static setCurrentRoute(currentRoute) {
    _channel.invokeListMethod("setCurrentRoute", currentRoute);
  }

  // Broadcasts

  static subscribeToTopic(topic) {
    _channel.invokeListMethod("subscribeToTopic", topic);
  }

  static unsubscribeFromTopic(topic) {
    _channel.invokeListMethod("unsubscribeFromTopic", topic);
  }

  static clearTopics() {
    _channel.invokeListMethod("clearTopics");
  }

  // Event Listener

  static void _onEvent(dynamic event) {
    if (event != null) {
      var eventMap = Map<String, dynamic>.from(event);
      var message = new GistMessage(eventMap["messageId"], eventMap["instanceId"], eventMap["queueId"]);
      var eventName = eventMap["eventName"];
      switch (eventName) {
        case "onMessageShown":
          eventListeners.forEach((listener) { listener?.onMessageShown(message); });
          break;
        case "onMessageDismissed":
          eventListeners.forEach((listener) { listener?.onMessageDismissed(message); });
          break;
        case "onError":
          eventListeners.forEach((listener) { listener?.onError(message); });
          break;
        case "onAction":
          var action = new GistAction(eventMap["currentRoute"], eventMap["action"]);
          eventListeners.forEach((listener) { listener?.onAction(message, action); });
          break;
      }
    }
  }

  static void _onError(Object error) {
    log("Event error");
  }

}

abstract class GistDelegate {
  onMessageShown(GistMessage message);
  onMessageDismissed(GistMessage message);
  onError(GistMessage message);
  onAction(GistMessage message, GistAction action);
}